@@ -6,7 +6,8 @@
 
 from __future__ import print_function
 
-import requests
+from requests import get
+
 import sparql
 import collections
 
@@ -22,6 +23,21 @@ LONG_TEXT = 0
 PREFIX    = 'http://it.dbpedia.org'
 ENDPOINT  = PREFIX + '/sparql'
 
+# FIXME: ugly hack ahead, needs a virtuoso >= 6.1.6 to remove it
+def unpack_row(row):
+    """
+    Transform row in python objects
+    needed because virtuoso < 6.1.6 has a RDF/XML bug on literals
+    otherwise similar to sparql.unpack_row()
+    """
+    pred, obj = sparql.unpack_row(row)
+    if isinstance(row[1], sparql.Literal):
+        try:
+            str(obj)
+        except UnicodeEncodeError:
+            obj = obj.encode('latin1').decode('utf8')
+    return pred, obj
+
 class Datatxt(object):
     """
     Main datatxt text parsers, instantiate a class passing deviation from
@@ -70,7 +86,7 @@ class Datatxt(object):
         u'text': u'L\'etimologia della parola che d\xc3\xa0 il nome alla citt\xc3\xa0 di Rovereto  deriva dal latino Roborem e trae dalla stessa radice della parola Robur: forza, robustezza, robusto. In Botanica la parola legno di rovere \xc3\xa8 correlata ad una specie di quercia indigena dell\'Europa, di minore altezza alla quercia ordinaria, ma che fornisce un legno durissimo: Quercus petraea. Nella toponomastica romana "Roboretum" significava selva di querce, albero che abbonda nella valle ed \xc3\xa8 effigie dello stemma comunale.'}
         """
         self.params['text'] = text
-        r = requests.get(self.api_url, params=self.params, headers=self.headers)
+        r = get(self.api_url, params=self.params, headers=self.headers)
         if r.status_code == 200:
             result = r.json
             for annotation in result['annotations']:
@@ -80,12 +96,12 @@ class Datatxt(object):
                     annotation[u'properties'] = self.properties(resource, wikilinks)
             return result
 
-    def query(self, query):
+    def sparql(self, query):
         result = self.service.query(query)
         statement = query.lower()
         if 'select' in statement:
             d = collections.defaultdict(list)
-            for pred, obj in (sparql.unpack_row(row) for row in result):
+            for pred, obj in (unpack_row(row) for row in result):
                 d[pred].append(obj)
             return dict(d)
         elif 'ask' in statement:
@@ -96,11 +112,20 @@ class Datatxt(object):
             query = "select * where {<%s> ?pred ?obj .}" % resource
         else:
             query = "select * where { graph <%s> {<%s> ?pred ?obj .}}" % (self.prefix, resource)
-        return self.query(query)
+        return self.sparql(query)
 
 if __name__ == '__main__':
     d = Datatxt()
-    for elem, lista in d.properties("http://it.dbpedia.org/resource/Rovereto").items():
-        print(elem, lista)
+    res = d.annotate("Sulle colline basaltiche, vicino a Rovereto si produce il Marzemino")
+    for ann in res['annotations']:
+        print("You are talking about <%s> (@pos %d, %d), with confidence %f" %
+                (ann['ref'], ann['start'], ann['end'], ann['rho']))
+        for key, vlist in ann['properties'].items():
+            if len(vlist) == 1:
+                values = vlist[0]
+            else:
+                values =  ' | '.join(str(vlist)).decode('utf8')
+
+            print(key, values)
 
 
